@@ -3,7 +3,8 @@ from hcloud.actions.client import BoundAction
 from hcloud.core.client import BoundModelBase, ClientEntityBase, GetEntityByNameMixin
 from hcloud.core.domain import add_meta_to_result
 
-from hcloud.firewalls.domain import Firewall, CreateFirewallResponse, FirewallRule, FirewallResource
+from hcloud.firewalls.domain import Firewall, CreateFirewallResponse, FirewallRule, FirewallResource, \
+    FirewallResourceLabelSelector
 
 
 class BoundFirewall(BoundModelBase):
@@ -22,9 +23,11 @@ class BoundFirewall(BoundModelBase):
             from hcloud.servers.client import BoundServer
             ats = []
             for a in applied_to:
-                if a["type"] == "server":
+                if a["type"] == FirewallResource.TYPE_SERVER:
                     ats.append(FirewallResource(type=a["type"], server=BoundServer(client._client.servers, a["server"],
                                                                                    complete=False)))
+                elif a["type"] == FirewallResource.TYPE_LABEL_SELECTOR:
+                    ats.append(FirewallResource(type=a["type"], label_selector=FirewallResourceLabelSelector(selector=a['label_selector']['selector'])))
             data['applied_to'] = ats
 
         super(BoundFirewall, self).__init__(client, data, complete)
@@ -238,6 +241,7 @@ class FirewallsClient(ClientEntityBase, GetEntityByNameMixin):
                name,  # type: str
                rules=None,  # type: Optional[List[FirewallRule]]
                labels=None,  # type: Optional[str]
+               resources=None,  # type: Optional[List[FirewallResource]]
                ):
         # type: (...) -> CreateFirewallResponse
         """Creates a new Firewall.
@@ -247,6 +251,7 @@ class FirewallsClient(ClientEntityBase, GetEntityByNameMixin):
         :param rules: List[:class:`FirewallRule <hcloud.firewalls.domain.FirewallRule>`] (optional)
         :param labels: Dict[str, str] (optional)
                User-defined labels (key-value pairs)
+        :param resources: List[:class:`FirewallResource <hcloud.firewalls.domain.FirewallResource>`] (optional)
         :return: :class:`CreateFirewallResponse <hcloud.firewalls.domain.CreateFirewallResponse>`
         """
 
@@ -260,16 +265,19 @@ class FirewallsClient(ClientEntityBase, GetEntityByNameMixin):
             data.update({"rules": []})
             for rule in rules:
                 data['rules'].append(rule.to_payload())
-
+        if resources is not None:
+            data.update({"apply_to": []})
+            for resource in resources:
+                data['apply_to'].append(resource.to_payload())
         response = self._client.request(url="/firewalls", json=data, method="POST")
 
-        action = None
-        if response.get('action') is not None:
-            action = BoundAction(self._client.actions, response['action'])
+        actions = []
+        if response.get('actions') is not None:
+            actions = [BoundAction(self._client.actions, _) for _ in response['actions']]
 
         result = CreateFirewallResponse(
             firewall=BoundFirewall(self, response['firewall']),
-            action=action
+            actions=actions
         )
         return result
 
@@ -329,7 +337,7 @@ class FirewallsClient(ClientEntityBase, GetEntityByNameMixin):
         """Applies one Firewall to multiple resources.
 
         :param firewall: :class:`BoundFirewall <hcloud.firewalls.client.BoundFirewall>` or  :class:`Firewall <hcloud.firewalls.domain.Firewall>`
-        :param rules: List[:class:`FirewallResource <hcloud.firewalls.domain.FirewallResource>`]
+        :param resources: List[:class:`FirewallResource <hcloud.firewalls.domain.FirewallResource>`]
         :return: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
         """
         data = {
@@ -347,7 +355,7 @@ class FirewallsClient(ClientEntityBase, GetEntityByNameMixin):
         """Removes one Firewall from multiple resources.
 
         :param firewall: :class:`BoundFirewall <hcloud.firewalls.client.BoundFirewall>` or  :class:`Firewall <hcloud.firewalls.domain.Firewall>`
-        :param rules: List[:class:`FirewallResource <hcloud.firewalls.domain.FirewallResource>`]
+        :param resources: List[:class:`FirewallResource <hcloud.firewalls.domain.FirewallResource>`]
         :return: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
         """
         data = {
