@@ -80,6 +80,10 @@ class FirewallRule:
     """Firewall Rule Protocol ICMP"""
     PROTOCOL_TCP = "tcp"
     """Firewall Rule Protocol TCP"""
+    PROTOCOL_ESP = "esp"
+    """Firewall Rule Protocol ESP"""
+    PROTOCOL_GRE = "gre"
+    """Firewall Rule Protocol GRE"""
 
     def __init__(
             self,
@@ -115,22 +119,30 @@ class FirewallResource:
            Type of resource referenced
     :param server: Optional[Server]
            Server the Firewall is applied to
+    :param label_selector: Optional[FirewallResourceLabelSelector]
+           Label Selector for Servers the Firewall should be applied to
     """
     __slots__ = (
         "type",
         "server",
+        "label_selector"
     )
 
     TYPE_SERVER = "server"
     """Firewall Used By Type Server"""
+    TYPE_LABEL_SELECTOR = "label_selector"
+    """Firewall Used By Type label_selector"""
 
     def __init__(
             self,
             type,  # type: str
             server=None,  # type: Optional[Server]
+            label_selector=None,  # type: Optional[FirewallResourceLabelSelector]
+
     ):
         self.type = type
         self.server = server
+        self.label_selector = label_selector
 
     def to_payload(self):
         payload = {
@@ -138,26 +150,39 @@ class FirewallResource:
         }
         if self.server is not None:
             payload.update({"server": {"id": self.server.id}})
+
+        if self.label_selector is not None:
+            payload.update({"label_selector": {"selector": self.label_selector.selector}})
         return payload
 
 
+class FirewallResourceLabelSelector(BaseDomain):
+    """FirewallResourceLabelSelector Domain
+
+    :param selector: str Target label selector
+    """
+
+    def __init__(self, selector=None):
+        self.selector = selector
+
+
 class CreateFirewallResponse(BaseDomain):
     """Create Firewall Response Domain
 
     :param firewall: :class:`BoundFirewall <hcloud.firewalls.client.BoundFirewall>`
            The Firewall which was created
-    :param action: :class:`BoundAction <hcloud.actions.client.BoundAction>`
+    :param actions: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
            The Action which shows the progress of the Firewall Creation
     """
     __slots__ = (
         "firewall",
-        "action"
+        "actions"
     )
 
     def __init__(
             self,
             firewall,  # type: BoundFirewall
-            action,  # type: BoundAction
+            actions,  # type: BoundAction
     ):
         self.firewall = firewall
-        self.action = action
+        self.actions = actions
