@@ -20,6 +20,7 @@ from hcloud.load_balancers.client import LoadBalancersClient
 from hcloud.load_balancer_types.client import LoadBalancerTypesClient
 
 from .__version__ import VERSION
+from .firewalls.client import FirewallsClient
 
 
 class APIException(Exception):
@@ -58,6 +59,7 @@ class Client(object):
         self._api_endpoint = api_endpoint
         self._application_name = application_name
         self._application_version = application_version
+        self._requests_session = requests.Session()
         self.poll_interval = poll_interval
 
         self.datacenters = DatacentersClient(self)
@@ -133,6 +135,12 @@ class Client(object):
         :type: :class:`LoadBalancerTypesClient <hcloud.load_balancer_types.client.LoadBalancerTypesClient>`
         """
 
+        self.firewalls = FirewallsClient(self)
+        """FirewallsClient Instance
+
+        :type: :class:`FirewallsClient <hcloud.firewalls.client.FirewallsClient>`
+        """
+
     def _get_user_agent(self):
         """Get the user agent of the hcloud-python instance with the user application name (if specified)
 
@@ -188,7 +196,7 @@ class Client(object):
         :return: Response
         :rtype: requests.Response
         """
-        response = requests.request(
+        response = self._requests_session.request(
             method,
             self._api_endpoint + url,
             headers=self._get_headers(),
