@@ -173,13 +173,13 @@ class NetworksClient(ClientEntityBase, GetEntityByNameMixin):
         :return: (List[:class:`BoundNetwork <hcloud.networks.client.BoundNetwork>`], :class:`Meta <hcloud.core.domain.Meta>`)
         """
         params = {}
-        if name:
+        if name is not None:
             params["name"] = name
-        if label_selector:
+        if label_selector is not None:
             params["label_selector"] = label_selector
-        if page:
+        if page is not None:
             params["page"] = page
-        if per_page:
+        if per_page is not None:
             params["per_page"] = per_page
 
         response = self._client.request(url="/networks", method="GET", params=params)
@@ -351,6 +351,8 @@ class NetworksClient(ClientEntityBase, GetEntityByNameMixin):
         }
         if subnet.ip_range is not None:
             data["ip_range"] = subnet.ip_range
+        if subnet.vswitch_id is not None:
+            data["vswitch_id"] = subnet.vswitch_id
 
         response = self._client.request(
             url="/networks/{network_id}/actions/add_subnet".format(network_id=network.id),
